/* Copyright (c) Eclipseina GmbH 2018
 * ALL RIGHTS RESERVED
 *
 * Redistribution or use of this source file or it's binary form,
 * with or without modification
 * is not permitted without the written permission of Eclipseina GmbH.
 *
 * Filename: ec32demodefines.h
 * Description:
 * Created on: 06.09.2018
 *
 * License key:358o167me81da0721nie5775s09pi2753lc77e93
 */


#ifndef SRC_DEMO_EC32DEMODEFINES_H_
#define SRC_DEMO_EC32DEMODEFINES_H_

/*
 *
 * macros for calling
 *
 */
#define ECLIB_RCV_FIX32(par) \
	s32 (par), \
	const s8 (par##_sf)

#define ECLIB_PASS_FIX32(par) \
	(par), \
	(par##_sf)

/*
 *  defines
 */
#define ECLIB_S32_NAN		(s32)-2147483648L   // (s32) 0x80000000		// = -2147483648
#define ECLIB_S32_POS_INF  	(s32) 2147483647 	// (s32) 0x7FFFFFFF		// =  2147483647
#define ECLIB_U32_POS_INF  	(u32) 4294967294 	// (s32) 0xFFFFFFFE		// =  4294967294
#define ECLIB_S32_NEG_INF	(s32)-2147483647 	// (s32) 0x80000001		// = -2147483647

#define ECLIB_BIT_SIZE_U64  64

#define ECLIB_S64_NAN		/*lint -e9048*/ (s64) 0x8000000000000000  	/*lint +e9048*/  // (s64)(-9223372036854775808)
#define ECLIB_S64_POS_INF  	(s64) 0x7FFFFFFFFFFFFFFF 	// (s64)9223372036854775807
#define ECLIB_U64_POS_INF  	/*lint -e9048*/	(u64) 0xFFFFFFFFFFFFFFFE 	/*lint +e9048*/	//
#define ECLIB_S64_NEG_INF	/*lint -e9048*/	(s64) 0x8000000000000001 	/*lint +e9048*/	// (s64)(-9223372036854775807)


#define ECLIB_BIT_SIZE_S32  31
#define ECLIB_BIT_SIZE_U32  32
#define ECLIB_BIT_SIZE_S64  63

#define ECLIB_SQRT_OF_S32_MAX		 	(u32) 46340 	// 46340^2 = 2147395600 < 2^31


// with this macro the precompiler gives only s16 values to the parameter par.
// par_float would be optimized for the relevant microcontroller
#define ECLIB_SET_FIX32(par, par_double) {	\
	s8 par_sf = (par##_sf); \
\
	if ((par_double) != (par_double)){ \
		(par) = ECLIB_S32_NAN; \
	} \
	else {\
		if (par_sf >= 0) { \
			if (((par_double) * (double) (1ull << par_sf)) > 2147483647.0F) { 	/* cut to S32_POS_INF if overflow */ \
				(par) = ECLIB_S32_POS_INF;	\
			}	\
			else if(((par_double) * (double) (1ull << par_sf)) < -2147483647.0F) { 	/* cut to S32_NEG_INF if underflow */	\
				(par) = ECLIB_S32_NEG_INF;	\
			}	\
			else {	/* convert to s32 */ \
				(par) = (s32) ((par_double) * (double) (1ull << par_sf));	\
			}	\
		} \
		else { /* par_sf < 0) */ \
			if (((par_double) / (double) (1ull << (-par_sf))) > 2147483647.0F) { 	/* cut to S32_POS_INF if overflow */ \
				(par) = ECLIB_S32_POS_INF;	\
			}	\
			else if(((par_double) / (double) (1ull << (-par_sf))) < -2147483647.0F) { 	/* cut to S32_NEG_INF if underflow */	\
				(par) = ECLIB_S32_NEG_INF;	\
			}	\
			else {	/* convert to s32 */ \
				(par) = (s32) ((par_double) / (double) (1ull << (-par_sf)));	\
			}	\
		} \
	} \
}


#endif /* SRC_DEMO_EC32DEMODEFINES_H_ */
